program test_get_1_outerm

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: new_unit, count_lines
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       nf95_gw_var
  use nr_util, only: pi, rad_to_deg, deg_to_rad
  use shapelib, only: shpt_polygon, shpfileobject, ftdouble, shpclose
  use shapelib_03, only: shp_create_03, dbf_add_field_03, &
       shp_append_object_03, dbf_write_attribute_03

  use derived_types, only: ssh_contour
  use get_1_outerm_m, only: get_1_outerm

  implicit none

  integer ncid, varid, unit, n, l
  real longitude(2) ! in rad
  real latitude(2) ! in rad
  real step(2) ! in rad
  real, allocatable:: ssh(:, :, :) ! (:, :, 1) sea-surface height, in m

  integer:: ind_targ_extr(2) = [19, 11]
  ! indices in the two dimensions of target extremum

  real:: innermost_level = 0.2933 ! level of innermost contour, for
                                  ! each extremum
  logical:: cyclone = .true.
  type(ssh_contour) outermost_contour
  TYPE(shpfileobject) shphandle
  integer field_number, shape_number

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the significant extrema, except the
  ! target extremum

  namelist /main_nml/ ind_targ_extr, innermost_level, cyclone

  !----------------------------------------------------------------

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  print *, 'Reading from "h.nc"...'
  call nf95_open("h.nc", nf90_nowrite, ncid)

  call nf95_inq_varid(ncid, "lon", varid)
  call nf95_get_var(ncid, varid, longitude)
  longitude = longitude * deg_to_rad

  call nf95_inq_varid(ncid, "lat", varid)
  call nf95_get_var(ncid, varid, latitude)
  latitude = latitude * deg_to_rad

  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_gw_var(ncid, varid, ssh)
  call nf95_close(ncid)

  step = [longitude(2) - longitude(1), latitude(2) - latitude(1)]

  print *, "Reading from outside_points.csv..."
  call new_unit(unit)
  open(unit, file = "outside_points.csv", status = "old", action = "read", &
       position = "rewind")
  call count_lines(unit, n)
  n = n - 2 ! 2 title lines
  allocate(outside_points(2, n))
  rewind(unit)
  read(unit, fmt = *)
  read(unit, fmt = *)

  do l = 1, n
     read(unit, fmt = *) outside_points(:, l)
  end do

  outside_points = outside_points * deg_to_rad
  close(unit)
  
  outermost_contour = get_1_outerm(cyclone, &
       coord_extr = [longitude(1), latitude(1)] + (ind_targ_extr - 1) * step, &
       innermost_level = innermost_level, outside_points = outside_points, &
       ssh = ssh(:, :, 1), corner = [longitude(1), latitude(1)], step = step)

  if (outermost_contour%closed) then
     print *, "Radius of disk of equal area: ", &
          sqrt(outermost_contour%area / pi) / 1e3, "km"

     call shp_create_03("test_get_1_outerm", shpt_polygon, shphandle)
     call dbf_add_field_03(field_number, shphandle, 'level', ftdouble, &
          nwidth = 13, ndecimals = 6)
     call shp_append_object_03(shape_number, shphandle, shpt_polygon, &
          outermost_contour%points * rad_to_deg)
     call dbf_write_attribute_03(shphandle, shape_number, ifield = 0, &
          fieldvalue = outermost_contour%ssh)
     CALL shpclose(shphandle)
     print *, 'Created shapefile "test_get_1_outerm".'
  else
     print *, "Could not find an outermost contour."
  end if

end program test_get_1_outerm
