#!/usr/bin/env python3

import netCDF4
from mpl_toolkits import basemap
from scipy import interpolate
import shapefile
import matplotlib.pyplot as plt
import numpy as np
import sys
import math

with netCDF4.Dataset("uv.nc") as f:
    u = f.variables["u"][:]
    v = f.variables["v"][:]
    lon = f.variables["lon"][:]
    lat = f.variables["lat"][:]

lon2d, lat2d = np.meshgrid(lon, lat)

lon_rad = lon * np.pi / 180
lat_rad = lat * np.pi / 180

values = np.stack((u, v), axis = -1)

sf = shapefile.Reader("test_set_max_speed")
p = sf.shape(0)
p = np.array(p.points)

lon_interp = p[:, 0]
lat_interp = p[:, 1]

lon_interp_rad = lon_interp[:- 1] / 180 * np.pi
lat_interp_rad = lat_interp[:- 1] / 180 * np.pi

xi = np.column_stack((lat_interp_rad, lon_interp_rad))
values_x = interpolate.interpn((lat_rad, lon_rad), values, xi)

ui = values_x[:, 0]
vi = values_x[:, 1]

lon_0 = float(input("longitude extremum = ? "))
lat_0 = float(input("latitude extremum = ? "))

lon_0_rad = lon_0 * np.pi/180
lat_0_rad = lat_0 * np.pi/180

m = basemap.Basemap(projection="stere", lon_0 = lon_0, lat_0 = lat_0,
                    llcrnrlon = lon[0], llcrnrlat = lat[0],
                    urcrnrlon = lon[- 1], urcrnrlat= lat[- 1])
u_rot, v_rot = m.rotate_vector(u, v, lon, lat)
ui_rot, vi_rot = m.rotate_vector(ui, vi, lon_interp[:- 1], lat_interp[:- 1])

x = np.cos(lat_0_rad) * (lon_interp_rad - lon_0_rad)
y = lat_interp_rad  - lat_0_rad

v_azim = (x * vi - y * ui) / np.sqrt(x**2 + y**2)
print("mean azimuthal speed = ", np.mean(v_azim), "m s-1")

plt.figure()
m.plot(lon_interp, lat_interp, latlon=True)
m.drawparallels(range(math.floor(lat[0]), math.ceil(lat[- 1])),
                labels = [1, 0, 0, 0])
m.drawmeridians(range(math.floor(lon[0]), math.ceil(lon[- 1])),
                labels = [0, 0, 0, 1])
m.quiver(lon_interp[:- 1], lat_interp[:- 1], ui_rot, vi_rot, latlon = True)
m.scatter(lon_0, lat_0, latlon = True)
plt.title("interpolated velocity")

plt.figure()
m.plot(lon_interp, lat_interp, latlon=True)
m.drawparallels(range(math.floor(lat[0]), math.ceil(lat[- 1])),
                labels = [1, 0, 0, 0])
m.drawmeridians(range(math.floor(lon[0]), math.ceil(lon[- 1])),
                labels = [0, 0, 0, 1])
m.quiver(lon2d, lat2d, u_rot, v_rot, latlon = True)
m.scatter(lon_0, lat_0, latlon = True)
plt.title("velocity at grid points")

plt.show()
