program test_nearby_extr

  ! Libraries:
  use jumble, only: new_unit
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_gw_var
  use nr_util, only: deg_to_rad, rad_to_deg
  use shapelib, only: shpfileobject, shpclose, shpobject, dbfreadattribute
  use shapelib_03, only: shp_open_03, shp_read_object_03

  use derived_types, only: eddy
  use nearby_extr_m, only: nearby_extr

  implicit none

  integer ncid, varid, ishape_last, ishape, i, l, unit
  integer int_attr ! integer attribute

  integer, allocatable:: extr_map(:, :)
  ! At a point of extremum SSH: identification number or this
  ! extremum. 0 at other points.

  type(eddy), allocatable:: list_vis(:) ! visible eddies at a given date
  TYPE(shpfileobject) hshp
  TYPE(shpobject) psobject

  real, allocatable:: nearby(:, :) ! (2, :) longitude and
  ! latitude, in rad, of extrema near the target extremum   

  !--------------------------------------------------------------------------

  call nf95_open("extr_map.nc", nf90_nowrite, ncid)
  call nf95_inq_varid(ncid, "extr_map", varid)
  call nf95_gw_var(ncid, varid, extr_map)
  call nf95_close(ncid)

  call new_unit(unit)
  open(unit, file = "ishape_last.txt", status = "old", action = "read", &
       position = "rewind")
  read(unit, fmt = *) ishape_last ! first date
  close(unit)

  call shp_open_03(hshp, "extremum", pszaccess = "rb")
  allocate(list_vis(ishape_last + 1))

  do ishape = 0, ishape_last
     call dbfreadattribute(hshp, ishape, ifield = 2, attr = i)

     call dbfreadattribute(hshp, ishape, ifield = 4, attr = int_attr)
     list_vis(i)%cyclone = int_attr == 1

     call dbfreadattribute(hshp, ishape, ifield = 5, attr = int_attr)
     list_vis(i)%valid = int_attr == 1

     call shp_read_object_03(hshp, ishape, psobject)
     list_vis(i)%coord_extr = [psobject%padfx(1), psobject%padfy(1)] &
          * deg_to_rad
  end do

  CALL shpclose(hshp)
  nearby = nearby_extr(extr_map, list_vis, i = 6)
  print *, "nearby:"

  do l = 1, size(nearby, 2)
     print *, nearby(:, l) * rad_to_deg
  end do

end program test_nearby_extr
