program test_read_snapshot

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn
  use mpi_f08, only: mpi_init, mpi_finalize
  use nr_util, only: deg_to_rad, assert
  use shapelib, only: shpclose
  use shapelib_03, only: dbf_read_attribute_03

  use derived_types, only: snapshot, shp_triplet
  use read_snapshot_m, only: read_snapshot
  use shp_triplet_open_m, only: shp_triplet_open
  use write_snapshot_m, only: write_snapshot

  implicit none

  character(len = :), allocatable:: shp_triplet_dir
  type(snapshot) s
  TYPE(shp_triplet) hshp
  integer k1, copy

  real:: corner(2) = [0.125, - 59.875]
  ! longitude and latitude of the corner of the whole grid in input
  ! NetCDF, in degrees

  real:: step(2) = [0.25, 0.25] ! longitude and latitude steps, in degrees
  logical periodic ! grid is periodic in longitude

  integer:: nlon = 120, nlat = 120
  ! size of ssh array in input NetCDF, assuming no repeated point if
  ! the grid is global

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ corner, step, nlon, nlat, dist_lim

  !-------------------------------------------------------------------------

  call mpi_init
  call get_command_arg_dyn(1, shp_triplet_dir, &
       "Required argument: SHP-triplet-directory")

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step(1) and nlon:
  periodic = nint(360. / step(1)) == nlon
  print *, "periodic = ", periodic
  if (periodic) call assert(2 * dist_lim * step(1) < 180., &
       "test_read_snapshot dist_lim")
  copy = merge(dist_lim, 0, periodic)

  call shp_triplet_open(hshp, trim(shp_triplet_dir), rank = 0)
  call dbf_read_attribute_03(k1, hshp%extremum, hshp%extr_date, ishape = 0)
  call read_snapshot(s, k1, hshp, corner * deg_to_rad, step * deg_to_rad, &
       nlon, nlat, copy, k1, ishape_last = [4])
  CALL shpclose(hshp%extremum)
  CALL shpclose(hshp%outermost)
  CALL shpclose(hshp%max_speed)

  call write_snapshot(s, corner, step, nlon, nlat, copy, k1)
  call mpi_finalize

end program test_read_snapshot
