program test_send_recv

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use ezmpi, only: ezmpi_bcast
  use jumble, only: get_command_arg_dyn, read_column
  use mpi_f08, only: mpi_init, mpi_finalize, MPI_Comm_rank, MPI_Comm_world, &
       MPI_Comm_size, mpi_abort, MPI_TAG_UB, MPI_Comm_get_attr, &
       MPI_ADDRESS_KIND, MPI_io, MPI_ANY_SOURCE
  use nr_util, only: deg_to_rad, assert
  use shapelib, only: shpclose
  use shapelib_03, only: dbf_read_attribute_03

  use derived_types, only: snapshot, shp_triplet
  use init_shapefiles_m, only: init_shapefiles
  use read_snapshot_m, only: read_snapshot
  use recv_snapshot_m, only: recv_snapshot
  use send_snapshot_m, only: send_snapshot
  use shp_triplet_open_m, only: shp_triplet_open
  use write_eddy_m, only: write_eddy
  use write_snapshot_m, only: write_snapshot

  implicit none

  character(len = :), allocatable:: shp_triplet_dir
  type(snapshot) s
  integer rank, n_proc, k1, copy, n_dates
  logical flag
  INTEGER(KIND=MPI_ADDRESS_KIND) attribute_val

  real:: corner(2) = [0.125, - 59.875]
  ! longitude and latitude of the corner of the whole grid, in degrees

  real:: step(2) = [0.25, 0.25] ! longitude and latitude steps, in degrees
  integer:: nlon = 120, nlat = 120

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ corner, step, nlon, nlat, dist_lim
  logical periodic ! grid is periodic in longitude
  TYPE(shp_triplet) hshp

  integer, allocatable:: ishape_last(:)
  ! shape index (0-based) in the triplet of shapefiles of the last
  ! shape at a given date index

  !---------------------------------------------------------------------

  call mpi_init
  call MPI_Comm_rank(MPI_Comm_world, rank)
  call MPI_Comm_size(MPI_Comm_world, n_proc)

  if (n_proc /= 2) then
     if (rank == 0) print *, "test_send_recv: exactly 2 processes required"
     call mpi_abort(MPI_Comm_world, errorcode = 1)
  end if

  call get_command_arg_dyn(1, shp_triplet_dir, &
       "Required argument: SHP-triplet-directory")

  if (rank == 0) then
     call MPI_Comm_get_attr(MPI_Comm_world, MPI_io, attribute_val, flag)
     call assert(flag, "test_send_recv MPI_Comm_get_attr MPI_io")
     call assert(attribute_val == MPI_ANY_SOURCE, &
          "test_send_recv MPI_io MPI_ANY_SOURCE")
     
     write(unit = error_unit, nml = main_nml)
     write(unit = error_unit, fmt = *) "Enter namelist main_nml."
     read(unit = *, nml = main_nml)
     write(unit = *, nml = main_nml)

     ! As we are requiring the grid spacing to be uniform, the value of
     ! "periodic" may be deduced from the values of step(1) and nlon:
     periodic = nint(360. / step(1)) == nlon
     print *, "periodic = ", periodic
     if (periodic) call assert(2 * dist_lim * step(1) < 180., &
          "test_send_recv dist_lim")
     copy = merge(dist_lim, 0, periodic)
     call read_column(ishape_last, &
          file = trim(shp_triplet_dir) // "/ishape_last.txt")
     n_dates = size(ishape_last)
  end if

  call ezmpi_bcast(corner, root = 0)
  call ezmpi_bcast(step, root = 0)
  call ezmpi_bcast(nlon, root = 0)
  call ezmpi_bcast(nlat, root = 0)
  call ezmpi_bcast(copy, root = 0)
  call ezmpi_bcast(n_dates, root = 0)
  if (rank == 1) allocate(ishape_last(n_dates))
  call ezmpi_bcast(ishape_last, root = 0)

  call shp_triplet_open(hshp, trim(shp_triplet_dir), rank)
  if (rank == 0) call dbf_read_attribute_03(k1, hshp%extremum, hshp%extr_date, &
       ishape = 0)
  call ezmpi_bcast(k1, root = 0)
  if (rank == 1) call read_snapshot(s, k1, hshp, corner * deg_to_rad, &
       step * deg_to_rad, nlon, nlat, copy, k1, ishape_last)
  CALL shpclose(hshp%extremum)
  CALL shpclose(hshp%outermost)
  CALL shpclose(hshp%max_speed)
  
  if (rank == 1) then
     call send_snapshot(s, dest = 0, tag = k1)
  else
     ! rank == 0
     call MPI_Comm_get_attr(MPI_Comm_world, MPI_TAG_UB, attribute_val, flag)
     call assert(flag, "test_send_recv MPI_Comm_get_attr MPI_TAG_UB")
     print *, "MPI_TAG_UB = ", attribute_val
     call recv_snapshot(s, nlon, nlat, copy, source = 1, tag = k1)
     call write_snapshot(s, corner, step, nlon, nlat, copy, k1)
  end if

  call mpi_finalize

end program test_send_recv
